@@ -13,7 +13,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * $Id: task_manager.c 3666 2008-03-26 18:40:19Z tobias $
+ * $Id: task_manager.c 4484 2008-10-27 11:13:33Z martin $
  */
 
 #include "task_manager.h"
@@ -145,44 +145,15 @@ struct private_task_manager_t {
  */
 static void flush(private_task_manager_t *this)
 {
-	task_t *task;
-	
 	this->queued_tasks->destroy_offset(this->queued_tasks, 
 										offsetof(task_t, destroy));
 	this->passive_tasks->destroy_offset(this->passive_tasks,
 										offsetof(task_t, destroy));
-	
-	/* emmit outstanding signals for tasks */
-	while (this->active_tasks->remove_last(this->active_tasks,
-										   (void**)&task) == SUCCESS)
-	{
-		switch (task->get_type(task))
-		{
-			case IKE_AUTH:
-				SIG(IKE_UP_FAILED, "establishing IKE_SA failed");
-				break;
-			case IKE_DELETE:
-				SIG(IKE_DOWN_FAILED, "IKE_SA deleted");
-				break;
-			case IKE_REKEY:
-				SIG(IKE_REKEY_FAILED, "rekeying IKE_SA failed");
-				break;
-			case CHILD_CREATE:
-				SIG(CHILD_UP_FAILED, "establishing CHILD_SA failed");
-				break;
-			case CHILD_DELETE:
-				SIG(CHILD_DOWN_FAILED, "deleting CHILD_SA failed");
-				break;
-			case CHILD_REKEY:
-				SIG(IKE_REKEY_FAILED, "rekeying CHILD_SA failed");
-				break;
-			default:
-				break;
-		}
-		task->destroy(task);
-	}
+	this->active_tasks->destroy_offset(this->active_tasks,
+										offsetof(task_t, destroy));
 	this->queued_tasks = linked_list_create();
 	this->passive_tasks = linked_list_create();
+	this->active_tasks = linked_list_create();
 }
 
 /**
@@ -648,6 +619,7 @@ static status_t build_response(private_task_manager_t *this, message_t *request)
 	DESTROY_IF(this->responding.packet);
 	status = this->ike_sa->generate_message(this->ike_sa, message,
 											&this->responding.packet);
+	charon->bus->message(charon->bus, message, FALSE);
 	message->destroy(message);
 	if (status != SUCCESS)
 	{
@@ -775,6 +747,8 @@ static status_t process_request(private_task_manager_t *this,
 							case UNACCEPTABLE_ADDRESSES:
 							case UNEXPECTED_NAT_DETECTED:
 							case COOKIE2:
+							case NAT_DETECTION_SOURCE_IP:
+							case NAT_DETECTION_DESTINATION_IP:
 								task = (task_t*)ike_mobike_create(
 														this->ike_sa, FALSE);
 								break;
@@ -865,6 +839,7 @@ static status_t process_message(private_task_manager_t *this, message_t *msg)
 	{
 		if (mid == this->responding.mid)
 		{
+			charon->bus->message(charon->bus, msg, TRUE);
 			if (process_request(this, msg) != SUCCESS)
 			{
 				flush(this);
