@@ -1,10 +1,3 @@
-/**
- * @file task.h
- * 
- * @brief Interface task_t.
- * 
- */
-
 /*
  * Copyright (C) 2007 Tobias Brunner
  * Copyright (C) 2006 Martin Willi
@@ -19,6 +12,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
+ *
+ * $Id: task.h 3666 2008-03-26 18:40:19Z tobias $
+ */
+
+/**
+ * @defgroup task task
+ * @{ @ingroup tasks
  */
 
 #ifndef TASK_H_
@@ -32,9 +32,7 @@ typedef struct task_t task_t;
 #include <encoding/message.h>
 
 /**
- * @brief Different kinds of tasks.
- * 
- * @ingroup tasks
+ * Different kinds of tasks.
  */
 enum task_type_t {
 	/** establish an unauthenticated IKE_SA */
@@ -47,8 +45,10 @@ enum task_type_t {
 	IKE_AUTHENTICATE,
 	/** AUTH_LIFETIME negotiation, RFC4478 */
 	IKE_AUTH_LIFETIME,
-	/** exchange certificates and requests */
-	IKE_CERT,
+	/** certificate processing before authentication (certreqs, cert parsing) */
+	IKE_CERT_PRE,
+	/** certificate processing after authentication (certs payload generation) */
+	IKE_CERT_POST,
 	/** Configuration payloads, virtual IP and such */
 	IKE_CONFIG,
 	/** rekey an IKE_SA */
@@ -59,10 +59,10 @@ enum task_type_t {
 	IKE_DELETE,
 	/** liveness check */
 	IKE_DPD,
-#ifdef P2P
-	/** handle P2P-NAT-T stuff */
-	IKE_P2P,
-#endif /* P2P */
+#ifdef ME
+	/** handle ME stuff */
+	IKE_ME,
+#endif /* ME */
 	/** establish a CHILD_SA within an IKE_SA */
 	CHILD_CREATE,
 	/** delete an established CHILD_SA */
@@ -77,7 +77,7 @@ enum task_type_t {
 extern enum_name_t *task_type_names;
 
 /**
- * @brief Interface for a task, an operation handled within exchanges.
+ * Interface for a task, an operation handled within exchanges.
  *
  * A task is an elemantary operation. It may be handled by a single or by
  * multiple exchanges. An exchange may even complete multiple tasks.
@@ -94,18 +94,12 @@ extern enum_name_t *task_type_names;
  * the task needs further build()/process() calls to complete, the manager
  * leaves the taks in the queue. A returned FAILED indicates a critical failure.
  * The manager closes the IKE_SA whenever a task returns FAILED.
- *
- * @b Constructors:
- *  - None, use implementations specific constructors
- * 
- * @ingroup tasks
  */
 struct task_t {
 
 	/**
-	 * @brief Build a request or response message for this task.
+	 * Build a request or response message for this task.
 	 * 
-	 * @param this 			calling object
 	 * @param message		message to add payloads to
 	 * @return
 	 * 						- FAILED if a critical error occured
@@ -115,9 +109,8 @@ struct task_t {
 	status_t (*build) (task_t *this, message_t *message);
 
 	/**
-	 * @brief Process a request or response message for this task.
+	 * Process a request or response message for this task.
 	 * 
-	 * @param this 			calling object
 	 * @param message		message to read payloads from
 	 * @return
 	 * 						- FAILED if a critical error occured
@@ -127,14 +120,12 @@ struct task_t {
 	status_t (*process) (task_t *this, message_t *message);
 
 	/**
-	 * @brief Get the type of the task implementation.
-	 * 
-	 * @param this 			calling object
+	 * Get the type of the task implementation.
 	 */
 	task_type_t (*get_type) (task_t *this);
 	
 	/**
-	 * @brief Migrate a task to a new IKE_SA.
+	 * Migrate a task to a new IKE_SA.
 	 *
 	 * After migrating a task, it goes back to a state where it can be
 	 * used again to initate an exchange. This is useful when a task
@@ -144,17 +135,14 @@ struct task_t {
 	 * try.
 	 * The ike_sa is the new IKE_SA this task belongs to and operates on.
 	 *
-	 * @param this 			calling object
 	 * @param ike_sa		new IKE_SA this task works for
 	 */
 	void (*migrate) (task_t *this, ike_sa_t *ike_sa);
 	
 	/**
-	 * @brief Destroys a task_t object.
-	 *
-	 * @param this 			calling object
+	 * Destroys a task_t object.
 	 */
 	void (*destroy) (task_t *this);
 };
 
-#endif /* TASK_H_ */
+#endif /* TASK_H_ @} */
