@@ -13,7 +13,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * $Id: gmp_rsa_private_key.c 4014 2008-05-23 19:23:04Z andreas $
+ * $Id: gmp_rsa_private_key.c 4345 2008-09-17 08:10:48Z martin $
  */
 
 #include <gmp.h>
@@ -191,6 +191,10 @@ static chunk_t rsadp(private_gmp_rsa_private_key_t *this, chunk_t data)
 	
 	decrypted.len = this->k;
 	decrypted.ptr = mpz_export(NULL, NULL, 1, decrypted.len, 1, 0, t1);
+	if (decrypted.ptr == NULL)
+	{
+		decrypted.len = 0;
+	}
 	
 	mpz_clear_randomized(t1);
 	mpz_clear_randomized(t2);
@@ -789,34 +793,37 @@ static gmp_rsa_private_key_t *build(private_builder_t *this)
  */
 static void add(private_builder_t *this, builder_part_t part, ...)
 {
-	va_list args;
+	if (!this->key)
+	{
+		va_list args;
+		chunk_t chunk;
 	
+		switch (part)
+		{
+			case BUILD_BLOB_ASN1_DER:
+			{
+				va_start(args, part);
+				chunk = va_arg(args, chunk_t);
+				this->key = load(chunk_clone(chunk));
+				va_end(args);
+				return;
+			}		
+			case BUILD_KEY_SIZE:
+			{
+				va_start(args, part);
+				this->key = generate(va_arg(args, u_int));
+				va_end(args);
+				return;
+			}
+			default:
+				break;
+		}
+	}
 	if (this->key)
 	{
-		DBG1("ignoring surplus build part %N", builder_part_names, part);
-		return;
-	}
-	
-	switch (part)
-	{
-		case BUILD_BLOB_ASN1_DER:
-		{
-			va_start(args, part);
-			this->key = load(va_arg(args, chunk_t));
-			va_end(args);
-			break;
-		}		
-		case BUILD_KEY_SIZE:
-		{
-			va_start(args, part);
-			this->key = generate(va_arg(args, u_int));
-			va_end(args);
-			break;
-		}
-		default:
-			DBG1("ignoring unsupported build part %N", builder_part_names, part);
-			break;
+		destroy((private_gmp_rsa_private_key_t*)this->key);
 	}
+	builder_cancel(&this->public);
 }
 
 /**
