@@ -23,10 +23,8 @@
 #define SHA1_HASHER_H_
 
 typedef struct sha1_hasher_t sha1_hasher_t;
-typedef struct sha1_keyed_prf_t sha1_keyed_prf_t;
 
 #include <crypto/hashers/hasher.h>
-#include <crypto/prfs/prf.h>
 
 /**
  * Implementation of hasher_t interface using the SHA1 algorithm.
@@ -39,17 +37,6 @@ struct sha1_hasher_t {
 	hasher_t hasher_interface;
 };
 
-/**
- * Implementation of prf_t interface using keyed SHA1 algorithm (used for EAP-AKA).
- */
-struct sha1_keyed_prf_t {
-	
-	/**
-	 * Implements prf_t interface.
-	 */
-	prf_t prf_interface;
-};
-
 /**
  * Creates a new sha1_hasher_t.
  *
@@ -58,12 +45,4 @@ struct sha1_keyed_prf_t {
  */
 sha1_hasher_t *sha1_hasher_create(hash_algorithm_t algo);
 
-/**
- * Creates a new sha1_keyed_prf_t.
- *
- * @param algo		algorithm, must be PRF_KEYED_SHA1
- * @return			sha1_keyed_prf_tobject
- */
-sha1_keyed_prf_t *sha1_keyed_prf_create(pseudo_random_function_t algo);
-
 #endif /*SHA1_HASHER_H_ @}*/
