@@ -13,7 +13,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * $Id: identification.c 4064 2008-06-13 15:10:01Z martin $
+ * $Id: identification.c 4344 2008-09-17 02:17:01Z andreas $
  */
 
 #define _GNU_SOURCE
@@ -915,6 +915,8 @@ static int print(FILE *stream, const struct printf_info *info,
 		case ID_FQDN:
 		case ID_RFC822_ADDR:
 		case ID_DER_ASN1_GN_URI:
+		case ID_EAP:
+		case ID_IETF_ATTR_STRING:
 			proper = sanitize_chunk(this->encoded);
 			snprintf(buf, sizeof(buf), "%.*s", proper.len, proper.ptr);
 			chunk_free(&proper);
@@ -1169,6 +1171,8 @@ identification_t *identification_create_from_encoding(id_type_t type, chunk_t en
 		case ID_PUBKEY_INFO_SHA1:
 		case ID_PUBKEY_SHA1:
 		case ID_CERT_DER_SHA1:
+		case ID_EAP:
+		case ID_IETF_ATTR_STRING:
 		default:
 			break;
 	}
